@@ -1,11 +1,13 @@
 import os
 import json
 import unittest
+import logging
 from typing import List, Tuple, Any
 
 from linkml_runtime.linkml_model.meta import SchemaDefinition, ClassDefinition
 from linkml_runtime.loaders.yaml_loader import YAMLLoader
 from linkml_runtime.utils.schemaview import SchemaView
+from linkml_runtime.utils.schemaops import roll_up, roll_down
 
 from tests.test_utils import INPUT_DIR
 
@@ -19,39 +21,39 @@ class SchemaViewTestCase(unittest.TestCase):
     def test_schemaview(self):
         # no import schema
         view = SchemaView(SCHEMA_NO_IMPORTS)
-        print(view.imports_closure())
+        logging.debug(view.imports_closure())
         assert len(view.imports_closure()) == 1
         all_cls = view.all_class()
-        print(f'n_cls = {len(all_cls)}')
+        logging.debug(f'n_cls = {len(all_cls)}')
 
         e = view.get_element('is current')
         assert list(view.annotation_dict('is current').values()) == ['bar']
-        print(view.annotation_dict('employed at'))
+        logging.debug(view.annotation_dict('employed at'))
         e = view.get_element('employed at')
-        print(e.annotations)
+        logging.debug(e.annotations)
         e = view.get_element('has employment history')
-        print(e.annotations)
+        logging.debug(e.annotations)
         #assert list(view.annotation_dict('employed at')[]
 
         if True:
             # this section is mostly for debugging
             for cn in all_cls.keys():
-                print(f'{cn} PARENTS = {view.class_parents(cn)}')
-                print(f'{cn} ANCS = {view.class_ancestors(cn)}')
-                print(f'{cn} CHILDREN = {view.class_children(cn)}')
-                print(f'{cn} DESCS = {view.class_descendants(cn)}')
-                print(f'{cn} SCHEMA = {view.in_schema(cn)}')
-                print(f'  SLOTS = {view.class_slots(cn)}')
+                logging.debug(f'{cn} PARENTS = {view.class_parents(cn)}')
+                logging.debug(f'{cn} ANCS = {view.class_ancestors(cn)}')
+                logging.debug(f'{cn} CHILDREN = {view.class_children(cn)}')
+                logging.debug(f'{cn} DESCS = {view.class_descendants(cn)}')
+                logging.debug(f'{cn} SCHEMA = {view.in_schema(cn)}')
+                logging.debug(f'  SLOTS = {view.class_slots(cn)}')
                 for sn in view.class_slots(cn):
                     slot = view.get_slot(sn)
                     if slot is None:
-                        print(f'NO SLOT: {sn}')
+                        logging.debug(f'NO SLOT: {sn}')
                     else:
-                        print(f'  SLOT {sn} R: {slot.range} U: {view.get_uri(sn)} ANCS: {view.slot_ancestors(sn)}')
+                        logging.debug(f'  SLOT {sn} R: {slot.range} U: {view.get_uri(sn)} ANCS: {view.slot_ancestors(sn)}')
                     induced_slot = view.induced_slot(sn, cn)
-                    print(f'    INDUCED {sn}={induced_slot}')
+                    logging.debug(f'    INDUCED {sn}={induced_slot}')
 
-        print(f'ALL = {view.all_element().keys()}')
+        logging.debug(f'ALL = {view.all_element().keys()}')
 
         # -- TEST ANCESTOR/DESCENDANTS FUNCTIONS --
 
@@ -80,7 +82,7 @@ class SchemaViewTestCase(unittest.TestCase):
 
         assert view.get_class('agent').class_uri == 'prov:Agent'
         assert view.get_uri('agent') == 'prov:Agent'
-        print(view.get_class('Company').class_uri)
+        logging.debug(view.get_class('Company').class_uri)
         #assert view.get_class('Company').class_uri == 'prov:Agent'
         assert view.get_uri('Company') == 'ks:Company'
 
@@ -94,7 +96,7 @@ class SchemaViewTestCase(unittest.TestCase):
             assert view.induced_slot('name', c).range == 'string'
         for c in ['Event', 'EmploymentEvent', 'MedicalEvent']:
             s = view.induced_slot('started at time', c)
-            print(f's={s.range} // c = {c}')
+            logging.debug(f's={s.range} // c = {c}')
             assert s.range == 'date'
             assert s.slot_uri == 'prov:startedAtTime'
         assert view.induced_slot('age in years', 'Person').minimum_value == 0
@@ -102,16 +104,67 @@ class SchemaViewTestCase(unittest.TestCase):
         assert view.induced_slot('type', 'FamilialRelationship').range == 'FamilialRelationshipType'
         assert view.induced_slot('related to', 'FamilialRelationship').range == 'Person'
 
+        a = view.get_class('activity')
+        self.assertCountEqual(a.exact_mappings, ['prov:Activity'])
+        logging.debug(view.get_mappings('activity',expand=True))
+        self.assertCountEqual(view.get_mappings('activity')['exact'], ['prov:Activity'])
+        self.assertCountEqual(view.get_mappings('activity', expand=True)['exact'], ['http://www.w3.org/ns/prov#Activity'])
+
         u = view.usage_index()
         for k, v in u.items():
-            print(f' {k} = {v}')
+            logging.debug(f' {k} = {v}')
+
+
+        # test methods also work for attributes
+        leaves = view.class_leaves()
+        logging.debug(f'LEAVES={leaves}')
+        assert 'MedicalEvent' in leaves
+        roots = view.class_roots()
+        logging.debug(f'ROOTS={roots}')
+        assert 'Dataset' in roots
+        ds_slots = view.class_slots('Dataset')
+        logging.debug(ds_slots)
+        assert len(ds_slots) == 3
+        self.assertCountEqual(['persons', 'companies', 'activities'], ds_slots)
+        for sn in ds_slots:
+            s = view.induced_slot(sn, 'Dataset')
+            logging.debug(s)
 
         #for e in view.all_element(imports=True):
-        #    print(view.annotation_dict(e))
-        #print(u)
+        #    logging.debug(view.annotation_dict(e))
+        #logging.debug(u)
+
+    def test_rollup_rolldown(self):
+        # no import schema
+        view = SchemaView(SCHEMA_NO_IMPORTS)
+        original_schema = view.copy_schema()
+        cn = 'Event'
+        roll_up(view, cn)
+        for s in view.class_induced_slots(cn):
+            logging.debug(s)
+        induced_slot_names = [s.name for s in view.class_induced_slots(cn)]
+        logging.debug(induced_slot_names)
+        self.assertCountEqual(['started at time', 'ended at time', 'is current', 'in location', 'employed at', 'married to'],
+                              induced_slot_names)
+        # check to make sure rolled-up classes are deleted
+        assert view.class_descendants(cn, reflexive=False) == []
+        roll_down(view, view.class_leaves())
+        for cn in view.all_class():
+            c = view.get_class(cn)
+            logging.debug(f'{cn}')
+            logging.debug(f'  {cn} SLOTS(i) = {view.class_slots(cn)}')
+            logging.debug(f'  {cn} SLOTS(d) = {view.class_slots(cn, direct=True)}')
+            self.assertCountEqual(view.class_slots(cn), view.class_slots(cn, direct=True))
+        assert 'Thing' not in view.all_class()
+        assert 'Person' not in view.all_class()
+        assert 'Adult' in view.all_class()
+
 
 
     def test_caching(self):
+        """
+        Determine if cache is reset after modifications made to schema
+        """
         s = SchemaDefinition(id='test', name='test')
         view = SchemaView(s)
         self.assertCountEqual([], view.all_class())
@@ -119,25 +172,39 @@ class SchemaViewTestCase(unittest.TestCase):
         self.assertCountEqual(['X'], view.all_class())
         view.add_class(ClassDefinition('Y'))
         self.assertCountEqual(['X', 'Y'], view.all_class())
-        # bypass view method
+        # bypass view method and add directly to schema;
+        # in general this is not recommended as the cache will
+        # not be updated
         view.schema.classes['Z'] = ClassDefinition('Z')
+        # as expected, the view doesn't know about Z
         self.assertCountEqual(['X', 'Y'], view.all_class())
+        # inform the view modifications have been made
         view.set_modified()
+        # should be in sync
         self.assertCountEqual(['X', 'Y', 'Z'], view.all_class())
+        # recommended way to make updates
         view.delete_class('X')
+        # cache will be up to date
         self.assertCountEqual(['Y', 'Z'], view.all_class())
+        view.add_class(ClassDefinition('W'))
+        self.assertCountEqual(['Y', 'Z', 'W'], view.all_class())
 
     def test_imports(self):
+        """
+        view should by default dynamically include imports chain
+        """
         view = SchemaView(SCHEMA_WITH_IMPORTS)
-        print(view.imports_closure())
+        logging.debug(view.imports_closure())
         self.assertCountEqual(['kitchen_sink', 'core', 'linkml:types'], view.imports_closure())
         for t in view.all_type().keys():
-            print(f'T={t} in={view.in_schema(t)}')
+            logging.debug(f'T={t} in={view.in_schema(t)}')
         assert view.in_schema('Person') == 'kitchen_sink'
         assert view.in_schema('id') == 'core'
         assert view.in_schema('name') == 'core'
         assert view.in_schema('activity') == 'core'
         assert view.in_schema('string') == 'types'
+        assert 'activity' in view.all_class()
+        assert 'activity' not in view.all_class(imports=False)
 
         for c in ['Company', 'Person', 'Organization', 'Thing']:
             assert view.induced_slot('id', c).identifier is True
@@ -146,7 +213,7 @@ class SchemaViewTestCase(unittest.TestCase):
             assert view.induced_slot('name', c).range == 'string'
         for c in ['Event', 'EmploymentEvent', 'MedicalEvent']:
             s = view.induced_slot('started at time', c)
-            print(f's={s.range} // c = {c}')
+            logging.debug(f's={s.range} // c = {c}')
             assert s.range == 'date'
             assert s.slot_uri == 'prov:startedAtTime'
         assert view.induced_slot('age in years', 'Person').minimum_value == 0
@@ -155,11 +222,11 @@ class SchemaViewTestCase(unittest.TestCase):
 
         assert view.get_class('agent').class_uri == 'prov:Agent'
         assert view.get_uri('agent') == 'prov:Agent'
-        print(view.get_class('Company').class_uri)
+        logging.debug(view.get_class('Company').class_uri)
         #assert view.get_class('Company').class_uri == 'prov:Agent'
         assert view.get_uri('Company') == 'ks:Company'
         assert view.get_uri('Company', expand=True) == 'https://w3id.org/linkml/tests/kitchen_sink/Company'
-        print(view.get_uri("TestClass"))
+        logging.debug(view.get_uri("TestClass"))
         assert view.get_uri('TestClass') == 'core:TestClass'
         assert view.get_uri('TestClass', expand=True) == 'https://w3id.org/linkml/tests/core/TestClass'
 
